<?php

namespace Drupal\spa\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\spa\SpaArgumentInterface;
use Drupal\spa\SpaArgumentPluginManager;
use Drupal\spa\SpaInterface;
use Drupal\spa\SpaLibraryInterface;
use Drupal\spa\SpaLibraryPluginManager;

/**
 * Defines the spa entity type.
 *
 * @ConfigEntityType(
 *   id = "spa",
 *   label = @Translation("SPA"),
 *   label_collection = @Translation("SPAs"),
 *   label_singular = @Translation("SPA"),
 *   label_plural = @Translation("SPAs"),
 *   label_count = @PluralTranslation(
 *     singular = "@count SPA",
 *     plural = "@count SPAs",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\spa\SpaListBuilder",
 *     "form" = {
 *       "add" = "Drupal\spa\Form\SpaForm",
 *       "edit" = "Drupal\spa\Form\SpaForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm"
 *     }
 *   },
 *   config_prefix = "spa",
 *   admin_permission = "administer spa",
 *   links = {
 *     "collection" = "/admin/structure/spa",
 *     "add-form" = "/admin/structure/spa/add",
 *     "edit-form" = "/admin/structure/spa/{spa}",
 *     "delete-form" = "/admin/structure/spa/{spa}/delete"
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   config_export = {
 *     "id",
 *     "iframe",
 *     "label",
 *     "description",
 *     "template",
 *     "arguments",
 *     "libraries"
 *   }
 * )
 */
class Spa extends ConfigEntityBase implements SpaInterface {

  /**
   * The spa ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The spa label.
   *
   * @var string
   */
  protected $label;

  /**
   * The spa status.
   *
   * @var bool
   */
  protected $status = TRUE;

  /**
   * Render as inside an iframe
   *
   * @var bool
   */
  protected $iframe;

  /**
   * The spa description.
   *
   * @var string
   */
  protected $description;

  /**
   * The SPA template.
   *
   * @var string
   */
  protected $template = '';

  /**
   * SPA Libraries
   *
   * @var SpaLibraryInterface[]
   */
  protected $libraries = [];

  /**
   * SPA Arguments
   *
   * @var SpaArgumentInterface[]
   */
  protected $arguments = [];

  /**
   * {@inheritdoc}
   */
  public function getArgumentConfig(): array {
    return $this->arguments ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function getIframeArguments(): array {
    return array_filter($this->getArguments(), function (SpaArgumentInterface $argument) {
      return $argument->passToIframe();
    });
  }

  /**
   * {@inheritdoc}
   */
  public function getArguments(): array {
    $arguments = [];
    foreach ($this->getArgumentConfig() as $argument_uuid => $config) {
      if ($argument = $this->getArgument($argument_uuid)) {
        $arguments[$argument_uuid] = $argument;
      }
    }

    return $arguments;
  }

  /**
   * {@inheritdoc}
   */
  public function getArgument(string $argument_uuid): ?SpaArgumentInterface {
    if ($argument_settings = $this->getArgumentConfig()[$argument_uuid]) {
      if (empty($argument_settings['id'])) {
        return NULL;
      }
      $argument_settings['uuid'] = $argument_uuid;
      /** @var SpaArgumentPluginManager $spa_argument_plugin_manager */
      $spa_argument_plugin_manager = \Drupal::service('plugin.manager.spa_argument');
      /** @var SpaArgumentInterface $argument */
      $argument = $spa_argument_plugin_manager->createInstance($argument_settings['id'], $argument_settings);
      $argument->setSpa($this);
      return $argument;
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraryConfig(): array {
    return $this->libraries ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries(): array {
    $libraries = [];
    foreach ($this->getLibraryConfig() as $library_uuid => $config) {
      $libraries[$library_uuid] = $this->getLibrary($library_uuid);
    }

    return $libraries;
  }

  /**
   * {@inheritdoc}
   */
  public function getLibrary(string $library_uuid): ?SpaLibraryInterface {
    if ($library_settings = $this->getLibraryConfig()[$library_uuid]) {
      if (empty($library_settings['id'])) {
        return NULL;
      }
      $library_settings['uuid'] = $library_uuid;
      /** @var SpaLibraryPluginManager $spa_library_plugin_manager */
      $spa_library_plugin_manager = \Drupal::service('plugin.manager.spa_library');
      /** @var SpaLibraryInterface $library */
      $library = $spa_library_plugin_manager->createInstance($library_settings['id'], $library_settings);
      $library->setSpa($this);
      return $library;
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getTemplate(): string {
    return $this->template ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function renderAsIframe(): bool {
    return $this->iframe ?? FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function libraryName(): string {
    return $this->id();
  }

  /**
   * {@inheritdoc}
   */
  public function libraryDefinition(): array {
    $definition = [];

    foreach ($this->getLibraries() as $library) {
      $library->setLibraryDefinition($definition);
    }

    return $definition;
  }

  /**
   * {@inheritdoc}
   */
  public function getContextDefinitions(): array {
    $definitions = [];
    foreach ($this->getArguments() as $argument) {
      if ($definition = $argument->getContextDefinition()) {
        $definitions[$argument->getArgumentName()] = $definition;
      }
    }

    return $definitions;
  }

}
