<?php

namespace Drupal\spa\Plugin\Block;

use Drupal\Component\Utility\Html;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Drupal\spa\Event\PassSpaArgumentEvent;
use Drupal\spa\SpaArgumentInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Exception\InvalidParameterException;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

/**
 * Provides the spa blocks.
 *
 * @Block(
 *   id = "spa",
 *   admin_label = @Translation("SPA"),
 *   category = @Translation("SPA"),
 *   deriver = "Drupal\spa\Plugin\Derivative\SpaBlockDeriver"
 * )
 */
class SpaBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Contracts\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * The SPA config entity.
   *
   * @var \Drupal\spa\Entity\spa|null
   */
  protected $spa = NULL;

  /**
   * Constructs a new BlockContentBlock.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The Plugin Block Manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Symfony\Contracts\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, EventDispatcherInterface $event_dispatcher) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->eventDispatcher = $event_dispatcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('event_dispatcher')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + ['arguments' => []];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    if (!$this->getEntity()) {
      $form['notice'] = [
        '#markup' => $this->t('Missing config!') ,
      ];

      return $form;
    }

    foreach ($this->initArguments($this->spa->getArguments()) as $argument) {
      $form = $argument->blockForm($form, $form_state);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    // Allow argument plugins to save their own settings.
    if (!$form_state->getErrors() && $this->getEntity()) {
      foreach ($this->spa->getArguments() as $uuid => $argument) {
        $this->configuration['arguments'][$uuid] = $argument->getBlockFormValue($form_state);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    if (!$this->getEntity()) {
      return ['content' => [
        '#markup' => $this->t('Missing config!') ,
      ]];
    }

    if ($this->spa->renderAsIframe()) {
      return $this->getIframe();
    }

    return $this->getInlineContent();
  }

  /**
   * Get the values from the argument plugins
   *
   * @param SpaArgumentInterface[] $arguments
   *   The arguments.
   * @return array
   *   The values.
   */
  protected function getArgumentValues(array $arguments): array {
    $argument_values = [];
    foreach ($arguments as $argument) {
      $event = new PassSpaArgumentEvent($this->spa, $argument);
      $this->eventDispatcher->dispatch($event, 'spa.pass_argument');
      $argument_values[$argument->getArgumentName()] = $event->getValue();
    }

    return $argument_values;
  }

  /**
   * Get the SPA inside an iframe.
   *
   * @param $argument_values
   *   Arguments that will be passed to the iframe.
   * @return array
   *   The iframe element
   */
  protected function getIframe() {
    $argument_values = $this->getArgumentValues($this->initArguments($this->spa->getIframeArguments()));
    try {
      $iframe_url = Url::fromRoute('spa.' . $this->spa->id() . '.iframe', $argument_values)->toString();
    }
    catch (InvalidParameterException $e) {
      return ['#markup' => $e->getMessage()];
    }

    $iframe_id = Html::getUniqueId($this->getPluginId());
    $build['content'] = [
      '#type' => 'inline_template',
      '#template' => '<iframe src="{{ url }}" id="{{ iframe_id }}" class="spa-iframe" width="100%"></iframe>',
      '#context' => [
        'url' => $iframe_url,
        'iframe_id' => $iframe_id
      ],
      '#attached' => [
        'library' => ['spa/iframe-resizer']
      ],
    ];

    return $build;
  }

  /**
   * Get the SPA element.
   *
   * @return array
   *   The SPA element
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getInlineContent() {
    $argument_values = $this->getArgumentValues($this->initArguments($this->spa->getArguments()));
    $build['content'] = [
      '#type' => 'inline_template',
      '#template' => $this->spa->getTemplate(),
      '#context' => $argument_values,
      '#attached' => [
        'library' => ['spa/' . $this->spa->libraryName()],
        'drupalSettings' => $argument_values,
      ],
    ];

    return $build;
  }

  /**
   * @param SpaArgumentInterface[] $arguments_instances
   * @return SpaArgumentInterface[]
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function initArguments(array $arguments_instances): array {
    if (!$this->getEntity()) {
      return [];
    }

    $arguments = [];
    foreach ($arguments_instances as $uuid => $argument) {
      $argument->setBlock($this);
      $argument->setSpa($this->spa);
      $arguments[$uuid] = $argument;
    }
    return $arguments;
  }

  /**
   * Loads the SPA entity of the block.
   *
   * @return \Drupal\spa\SpaInterface|null
   *   The block content entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getEntity() {
    if (!isset($this->spa)) {
      $id = $this->getDerivativeId();
      $this->spa = $this->entityTypeManager->getStorage('spa')->load($id);
    }

    return $this->spa;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    $cache_tags = parent::getCacheTags();
    if ($spa = $this->getEntity()) {
      $cache_tags = Cache::mergeTags(parent::getCacheTags(), $spa->getCacheTags());
    }
    return $cache_tags;
  }

}
