<?php

namespace Drupal\spa;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\libraries_ui\LibrariesUiService;
use Drupal\multivalue_form_element\Element\MultiValue;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * Base class for spa_library plugins.
 */
abstract class SpaLibraryPluginBase extends PluginBase implements SpaLibraryInterface, ContainerFactoryPluginInterface {
  use StringTranslationTrait;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('module_handler')
    );
  }

  /**
   * SpaLibraryPluginBase constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param ModuleHandlerInterface $module_handler
   *   The module handler service.
   *
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ModuleHandlerInterface $module_handler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->setConfiguration($configuration);
    $this->moduleHandler = $module_handler;
  }

  /**
   * The spa config entity
   *
   * @var SpaInterface
   */
  protected $spa;

  /**
   * Setter for the spa config entity.
   *
   * @param SpaInterface $spa
   *   The SPA config entity.
   */
  public function setSpa(SpaInterface $spa) {
    $this->spa = $spa;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['dependencies'] = [
      '#type' => 'multivalue',
      '#cardinality' => MultiValue::CARDINALITY_UNLIMITED,
      '#title' => $this->t('Dependencies'),
      '#add_more_label' => $this->t('Add a dependency'),
      'library' => [
        '#type' => 'textfield',
        '#title' => $this->t('Library'),
      ],
      '#default_value' => $this->configuration['dependencies'],
    ];

    // If library_ui is enabled we can provide a library select.
    if ($this->moduleHandler->moduleExists('libraries_ui')) {
      $form['dependencies']['library']['#type'] = 'select';
      $form['dependencies']['library']['#options'] = $this->getLibraryOptions();
      $form['dependencies']['library']['#empty_option'] = $this->t('None');
    }

    return $form;
  }

  /**
   * Get all library options.
   *
   * @return array
   */
  protected function getLibraryOptions(): array {
    /** @var LibrariesUiService $libraries_ui_service */
    $libraries_ui_service = \Drupal::service('libraries_ui');
    $options = [];
    foreach ($libraries_ui_service->getAllLibraries() as $module_name => $module_libraries) {
      foreach (array_keys($module_libraries) as $libaray_name) {
        $libaray = $module_name . '/' . $libaray_name;
        $options[$libaray] = $libaray;
      }
    }

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'dependencies' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = NestedArray::mergeDeep($this->defaultConfiguration(), $configuration);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    // Cast the label to a string since it is a TranslatableMarkup object.
    return (string) $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function getContent(): Response {
    return new Response('Not found', 404);
  }

  /**
   * {@inheritdoc}
   */
  public function getDependencies(): array {
    return $this->configuration['dependencies'] ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function setLibraryDefinition(&$definition): void {
    if (!isset($definition['dependencies'])) {
      $definition['dependencies'] = [];
    }

    if ($dependencies = $this->getDependencies()) {
      $libraries = array_column($dependencies, 'library');
      $definition['dependencies'] = array_unique(array_merge($definition['dependencies'], $libraries));
    }
  }

}
